'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.getDefaultValidators = getDefaultValidators;
exports.combineValidators = combineValidators;
exports.validate = validate;

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var INTEGER_REGEX = /^-?\d+?$/;
var FLOAT_REGEX = /^-?\d+(\.\d+)?$/;
var EMAIL_REGEX = /\S+@\S+\.\S+/;

var Validator = exports.Validator = function () {
    function Validator(name) {
        _classCallCheck(this, Validator);

        this.name = name;
    }

    _createClass(Validator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return true;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return true;
        }
    }, {
        key: 'createValidationError',
        value: function createValidationError(value, files, params) {
            var error = {
                key: this.name
            };
            if (Object.keys(params).length || value || files && files.length) {
                error.params = _extends({}, params);
                if (value) error.params.value = value;
                if (files && files.length) error.params.files = files;
            }
            return error;
        }
    }]);

    return Validator;
}();

var CustomValidator = exports.CustomValidator = function (_Validator) {
    _inherits(CustomValidator, _Validator);

    function CustomValidator() {
        _classCallCheck(this, CustomValidator);

        return _possibleConstructorReturn(this, (CustomValidator.__proto__ || Object.getPrototypeOf(CustomValidator)).apply(this, arguments));
    }

    _createClass(CustomValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            var _this2 = this;

            return !!value && props.validate && (props.validate === this.name || !!props.validate.find(function (_val) {
                return _val === _this2.name;
            }));
        }
    }]);

    return CustomValidator;
}(Validator);

var RequiredValidator = exports.RequiredValidator = function (_Validator2) {
    _inherits(RequiredValidator, _Validator2);

    function RequiredValidator() {
        _classCallCheck(this, RequiredValidator);

        return _possibleConstructorReturn(this, (RequiredValidator.__proto__ || Object.getPrototypeOf(RequiredValidator)).call(this, 'required'));
    }

    _createClass(RequiredValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!props.required;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            if (props.type === 'file') {
                return !!files && !!files.length;
            } else {
                return !!value;
            }
        }
    }]);

    return RequiredValidator;
}(Validator);

var PatternValidator = exports.PatternValidator = function (_Validator3) {
    _inherits(PatternValidator, _Validator3);

    function PatternValidator() {
        _classCallCheck(this, PatternValidator);

        return _possibleConstructorReturn(this, (PatternValidator.__proto__ || Object.getPrototypeOf(PatternValidator)).call(this, 'pattern'));
    }

    _createClass(PatternValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && !!props.pattern;
        }
    }, {
        key: 'getRegExp',
        value: function getRegExp(props) {
            if (props.pattern instanceof RegExp) {
                return props.pattern;
            } else {
                return new RegExp(props.pattern);
            }
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            var pattern = this.getRegExp(props);
            return !pattern.test(value) ? { pattern: pattern } : true;
        }
    }]);

    return PatternValidator;
}(Validator);

var EmailValidator = exports.EmailValidator = function (_Validator4) {
    _inherits(EmailValidator, _Validator4);

    function EmailValidator() {
        _classCallCheck(this, EmailValidator);

        return _possibleConstructorReturn(this, (EmailValidator.__proto__ || Object.getPrototypeOf(EmailValidator)).call(this, 'email'));
    }

    _createClass(EmailValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.type === 'email';
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return EMAIL_REGEX.test(value);
        }
    }]);

    return EmailValidator;
}(Validator);

var IntegerValidator = exports.IntegerValidator = function (_Validator5) {
    _inherits(IntegerValidator, _Validator5);

    function IntegerValidator() {
        _classCallCheck(this, IntegerValidator);

        return _possibleConstructorReturn(this, (IntegerValidator.__proto__ || Object.getPrototypeOf(IntegerValidator)).call(this, 'integer'));
    }

    _createClass(IntegerValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.type === 'number' && props.integer;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return INTEGER_REGEX.test(value);
        }
    }]);

    return IntegerValidator;
}(Validator);

var FloatValidator = exports.FloatValidator = function (_Validator6) {
    _inherits(FloatValidator, _Validator6);

    function FloatValidator() {
        _classCallCheck(this, FloatValidator);

        return _possibleConstructorReturn(this, (FloatValidator.__proto__ || Object.getPrototypeOf(FloatValidator)).call(this, 'float'));
    }

    _createClass(FloatValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.type === 'number' && !props.integer;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return FLOAT_REGEX.test(value);
        }
    }]);

    return FloatValidator;
}(Validator);

var MinLengthValidator = exports.MinLengthValidator = function (_Validator7) {
    _inherits(MinLengthValidator, _Validator7);

    function MinLengthValidator() {
        _classCallCheck(this, MinLengthValidator);

        return _possibleConstructorReturn(this, (MinLengthValidator.__proto__ || Object.getPrototypeOf(MinLengthValidator)).call(this, 'minLength'));
    }

    _createClass(MinLengthValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.minLength !== undefined && props.minLength !== null;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return (value + '').length < +props.minLength ? { length: (value + '').length, minLength: +props.minLength } : true;
        }
    }]);

    return MinLengthValidator;
}(Validator);

var MaxLengthValidator = exports.MaxLengthValidator = function (_Validator8) {
    _inherits(MaxLengthValidator, _Validator8);

    function MaxLengthValidator() {
        _classCallCheck(this, MaxLengthValidator);

        return _possibleConstructorReturn(this, (MaxLengthValidator.__proto__ || Object.getPrototypeOf(MaxLengthValidator)).call(this, 'maxLength'));
    }

    _createClass(MaxLengthValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.maxLength !== undefined && props.maxLength !== null;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return (value + '').length > +props.maxLength ? { length: (value + '').length, maxLength: props.maxLength } : true;
        }
    }]);

    return MaxLengthValidator;
}(Validator);

var MinValidator = exports.MinValidator = function (_Validator9) {
    _inherits(MinValidator, _Validator9);

    function MinValidator() {
        _classCallCheck(this, MinValidator);

        return _possibleConstructorReturn(this, (MinValidator.__proto__ || Object.getPrototypeOf(MinValidator)).call(this, 'min'));
    }

    _createClass(MinValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.type === 'number' && props.min !== undefined && props.min !== null;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return +value < +props.min ? { min: +props.min } : true;
        }
    }]);

    return MinValidator;
}(Validator);

var MaxValidator = exports.MaxValidator = function (_Validator10) {
    _inherits(MaxValidator, _Validator10);

    function MaxValidator() {
        _classCallCheck(this, MaxValidator);

        return _possibleConstructorReturn(this, (MaxValidator.__proto__ || Object.getPrototypeOf(MaxValidator)).call(this, 'max'));
    }

    _createClass(MaxValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && props.type === 'number' && props.max !== undefined && props.max !== null;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            return +value > +props.max ? { max: +props.max } : true;
        }
    }]);

    return MaxValidator;
}(Validator);

var MatchValidator = exports.MatchValidator = function (_Validator11) {
    _inherits(MatchValidator, _Validator11);

    function MatchValidator() {
        _classCallCheck(this, MatchValidator);

        return _possibleConstructorReturn(this, (MatchValidator.__proto__ || Object.getPrototypeOf(MatchValidator)).call(this, 'match'));
    }

    _createClass(MatchValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return !!value && !!props.match && formCtrl && formCtrl.values && formCtrl.fields[props.match];
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            var matchValue = formCtrl.values[props.match];
            return value !== matchValue ? { match: props.match } : true;
        }
    }]);

    return MatchValidator;
}(Validator);

var FileAcceptValidator = exports.FileAcceptValidator = function (_Validator12) {
    _inherits(FileAcceptValidator, _Validator12);

    function FileAcceptValidator() {
        _classCallCheck(this, FileAcceptValidator);

        return _possibleConstructorReturn(this, (FileAcceptValidator.__proto__ || Object.getPrototypeOf(FileAcceptValidator)).call(this, 'accept'));
    }

    _createClass(FileAcceptValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return props.type === 'file' && !!props.accept && files && files.length;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            var unmatchedFiles = files.filter(function (file) {
                var matchedAccepts = props.accept.replace(/ /g, '').split(',').filter(function (mimetype) {
                    return new RegExp('^' + mimetype.replace(/\*/g, '.*') + '$').test(file.type);
                });
                return matchedAccepts.length === 0;
            });
            return unmatchedFiles.length > 0 ? { files: unmatchedFiles, accept: props.accept } : true;
        }
    }]);

    return FileAcceptValidator;
}(Validator);

var FileExtensionValidator = exports.FileExtensionValidator = function (_Validator13) {
    _inherits(FileExtensionValidator, _Validator13);

    function FileExtensionValidator() {
        _classCallCheck(this, FileExtensionValidator);

        return _possibleConstructorReturn(this, (FileExtensionValidator.__proto__ || Object.getPrototypeOf(FileExtensionValidator)).call(this, 'extensions'));
    }

    _createClass(FileExtensionValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return props.type === 'file' && props.extensions && props.extensions.length && files && files.length;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            var unmatchedFiles = files.filter(function (file) {
                var matchedAccepts = props.extensions.filter(function (extension) {
                    var regex = new RegExp('\\.' + extension.replace(/\./, '') + '$', 'i');
                    return regex.test(file.name);
                });
                return matchedAccepts.length === 0;
            });
            return unmatchedFiles.length > 0 ? { files: unmatchedFiles, extensions: props.extensions } : true;
        }
    }]);

    return FileExtensionValidator;
}(Validator);

var FileMaxSizeValidator = exports.FileMaxSizeValidator = function (_Validator14) {
    _inherits(FileMaxSizeValidator, _Validator14);

    function FileMaxSizeValidator() {
        _classCallCheck(this, FileMaxSizeValidator);

        return _possibleConstructorReturn(this, (FileMaxSizeValidator.__proto__ || Object.getPrototypeOf(FileMaxSizeValidator)).call(this, 'maxSize'));
    }

    _createClass(FileMaxSizeValidator, [{
        key: 'shouldValidate',
        value: function shouldValidate(formCtrl, props, value, files) {
            return props.type === 'file' && +props.maxSize > 0 && files && files.length;
        }
    }, {
        key: 'validate',
        value: function validate(formCtrl, props, value, files) {
            var unmatchedFiles = files.filter(function (file) {
                return file.size > +props.maxSize;
            });
            return unmatchedFiles.length > 0 ? { files: unmatchedFiles, maxSize: +props.maxSize } : true;
        }
    }]);

    return FileMaxSizeValidator;
}(Validator);

function getDefaultValidators() {

    var validators = [];
    validators.push(new RequiredValidator());
    validators.push(new PatternValidator());
    validators.push(new EmailValidator());
    validators.push(new IntegerValidator());
    validators.push(new FloatValidator());
    validators.push(new MinLengthValidator());
    validators.push(new MaxLengthValidator());
    validators.push(new MinValidator());
    validators.push(new MaxValidator());
    validators.push(new MatchValidator());
    validators.push(new FileAcceptValidator());
    validators.push(new FileExtensionValidator());
    validators.push(new FileMaxSizeValidator());
    return validators;
}

function combineValidators() {
    var customValidators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];


    var validators = getDefaultValidators();

    var _customValidators = customValidators;
    if (!Array.isArray(_customValidators)) {
        _customValidators = [_customValidators];
    }

    _customValidators.forEach(function (validator) {
        return validators.push(validator);
    });

    return validators;
}

function validate() {
    var validators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var formCtrl = arguments[1];
    var props = arguments[2];
    var value = arguments[3];
    var files = arguments[4];

    var matchedValidators = validators.filter(function (validator) {
        return validator.shouldValidate(formCtrl, props, value, files);
    });
    if (matchedValidators && matchedValidators.length) {
        var errors = [];
        matchedValidators.forEach(function (validator) {
            var result = validator.validate(formCtrl, props, value, files);
            if (typeof result === 'boolean') {
                if (!result) {
                    errors.push(validator.createValidationError(value, files, {}));
                }
            } else {
                errors.push(validator.createValidationError(value, files, result));
            }
        });
        return errors;
    }
    return null;
}